/*
 * Copyright (c) 1985, 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      @(#)math.h      5.8 (Berkeley) 4/2/91
 */

#ifndef _MATH_H_
#define _MATH_H_

#include <float.h>

#ifndef BITS
#define BITS(type)      (8 * (int)sizeof(type))
#endif

#define M_E             2.7182818284590452354   /* e */
#define M_LOG2E         1.4426950408889634074   /* log 2e */
#define M_LOG10E        0.43429448190325182765  /* log 10e */
#define M_LN2           0.69314718055994530942  /* log e2 */
#define M_LN10          2.30258509299404568402  /* log e10 */
#define M_PI            3.14159265358979323846  /* pi */
#define M_PI_2          1.57079632679489661923  /* pi/2 */
#define M_PI_4          0.78539816339744830962  /* pi/4 */
#define M_1_PI          0.31830988618379067154  /* 1/pi */
#define M_2_PI          0.63661977236758134308  /* 2/pi */
#define M_2_SQRTPI      1.12837916709551257390  /* 2/sqrt(pi) */
#define M_SQRT2         1.41421356237309504880  /* sqrt(2) */
#define M_SQRT1_2       0.70710678118654752440  /* 1/sqrt(2) */

#include <sys/cdefs.h>

#define HUGE_VAL        1e500			/* IEEE: positive infinity */

#if !defined(_ANSI_SOURCE) && !defined(_POSIX_SOURCE)
__BEGIN_DECLS
extern const float		__inff;
extern const float		__nanf;
__END_DECLS
#define HUGE_VALF       1e50f			/* C99: positive infinity */
#define INFINITY        __inff		/* IEEE positive infinity.  */
#define NAN             __nanf		/* IEEE Not A Number.  */
#define HUGE            MAXFLOAT
#endif

#define X_TLOSS         1.41484755040568800000e+16 /* X_TLOSS = pi*2**52 */

#define DOMAIN          1
#define SING            2
#define OVERFLOW        3
#define UNDERFLOW       4
#define TLOSS           5
#define PLOSS           6

__BEGIN_DECLS
double acos __P((double));
double asin __P((double));
double atan __P((double));
double atan2 __P((double, double));
double cos __P((double));
double sin __P((double));
double tan __P((double));

double cosh __P((double));
double sinh __P((double));
double tanh __P((double));

double exp __P((double));
double frexp __P((double, int *));
double ldexp __P((double, int));
double log __P((double));
double log10 __P((double));
double modf __P((double, double *));

double pow __P((double, double));
double sqrt __P((double));

double ceil __P((double));
double fabs __P((double));
double floor __P((double));
double fmod __P((double, double));

double round __P((double));
long int lround __P((double));

double erf __P((double));
double erfc __P((double));
double gamma __P((double));
double hypot __P((double, double));
double j0 __P((double));
double j1 __P((double));
double jn __P((int, double));
double lgamma __P((double));
double y0 __P((double));
double y1 __P((double));
double yn __P((int, double));
double acosh __P((double));
double asinh __P((double));
double atanh __P((double));
double cbrt __P((double));
double logb __P((double));
double nextafter __P((double, double));
double remainder __P((double, double));
double remquo __P((double, double, int *));
double trunc __P((double));
double infinity __P((void));
double nan __P((const char *tagp));
#ifdef _SCALB_INT
double scalb __P((double, int));
#else
double scalb __P((double, double));
#endif

struct exception {
	int type;
	char *name;
	double arg1;
	double arg2;
	double retval;
};

#ifdef __cplusplus
int matherr __P((struct __exception *__exc)) throw();
#else
int matherr __P((struct exception *__exc));
#endif

__END_DECLS

/*
 * C99 Stuff
 */

#if !defined(_ANSI_SOURCE) && !defined(_POSIX_SOURCE)

enum {
  FP_NAN,
  FP_INFINITE,
  FP_ZERO,
  FP_SUBNORMAL,
  FP_NORMAL
};

#define MATH_ERRNO     1	/* errno set by math functions.  */
#define MATH_ERREXCEPT 2	/* Exceptions raised by math functions.  */

typedef float float_t;   /* `float' expressions are evaluated as `float'. */
typedef double double_t; /* `double' expressions are evaluated as `double'. */

/* The values returned by `ilogb' for 0 and NaN respectively.  */
#define FP_ILOGB0    (-2147483647)
#define FP_ILOGBNAN  (2147483647)

#ifndef FP_FAST_FMA
#define FP_FAST_FMA 1	/* We've got a fast C99 fma() with no intermediate rounding! */
#endif

#ifndef FP_FAST_FMAF
#define FP_FAST_FMAF 1	/* We've got a fast C99 fmaf() with no intermediate rounding! */
#endif

__BEGIN_DECLS
float fabsf __P((float));
float acosf __P((float));
float asinf __P((float));
float atanf __P((float));
float atan2f __P((float, float));
float cosf __P((float));
float sinf __P((float));
float tanf __P((float));
float coshf __P((float));
float sinhf __P((float));
float tanhf __P((float));
float expf __P((float));
float frexpf __P((float, int *));
float ldexpf __P((float, int));
float logf __P((float));
float log10f __P((float));
float modff __P((float, float *));
float powf __P((float, float));
float sqrtf __P((float));
float ceilf __P((float));
float floorf __P((float));
float fmodf __P((float, float));
float erff __P((float));
float erfcf __P((float));
float gammaf __P((float));
float hypotf __P((float, float));
float lgammaf __P((float));
float acoshf __P((float));
float asinhf __P((float));
float atanhf __P((float));
float cbrtf __P((float));
float logbf __P((float));
float remainderf __P((float, float));
float scalbnf __P((float, int));
float rintf __P((float));
long int lrintf __P((float));
int ilogbf __P((float));
float copysignf __P((float, float));
float expm1f __P((float));
float log1pf __P((float));
float roundf __P((float));
long int lroundf __P((float));
float remquof __P((float, float, int *));
float truncf __P((float));
float infinityf __P((void));
float nanf __P((const char *tagp));
__END_DECLS

__BEGIN_DECLS
int __fpclassify __P((double));
int __signbit __P((double));
int __finite __P((double));
int __isnan __P((double));
int __isinf __P((double));
__END_DECLS

__BEGIN_DECLS
int __fpclassifyf __P((float));
int __signbitf __P((float));
int __finitef __P((float));
int __isnanf __P((float));
int __isinff __P((float));
__END_DECLS

__BEGIN_DECLS
double fdim __P((double, double));
double fmax __P((double, double));
double fmin __P((double, double));
double fma __P((double, double, double));
__END_DECLS

__BEGIN_DECLS
float fdimf __P((float, float));
float fmaxf __P((float, float));
float fminf __P((float, float));
float fmaf __P((float, float, float));
__END_DECLS

#if 0
__BEGIN_DECLS
int fpclassify __P((double));
int signbit __P((double));
int finite __P((double));
int isnan __P((double));
int isinf __P((double));
__END_DECLS

__BEGIN_DECLS
int fpclassifyf __P((float));
int signbitf __P((float));
int finitef __P((float));
int isnanf __P((float));
int isinff __P((float));
__END_DECLS
#endif

#if !defined(_LIBM_BUILD)

/* Return number of classification appropriate for X.  */
#define fpclassify(x) \
	(sizeof (x) == sizeof (float) ? __fpclassifyf (x) : __fpclassify (x))

/* Return nonzero value if sign of X is negative.  */
#define signbit(x) \
	(sizeof (x) == sizeof (float) ? __signbitf (x) : __signbit (x))

/* Return nonzero value if X is not +-Inf or NaN.  */
#define isfinite(x) \
	(sizeof (x) == sizeof (float) ? __finitef (x) : __finite (x))

/* Return nonzero value if X is neither zero, subnormal, Inf, nor NaN.  */
#define isnormal(x) (fpclassify (x) == FP_NORMAL)

/* Return nonzero value if X is a NaN. */
#define isnan(x) \
	(sizeof (x) == sizeof (float) ? __isnanf (x) : __isnan (x))

/* Return nonzero value is X is positive or negative infinity.  */
#define isinf(x) \
	(sizeof (x) == sizeof (float) ? __isinff (x) : __isinf (x))

#endif /* !_LIBM_BUILD */

#endif

/*
 * IEEE Test Vector
 */
double significand __P((double));

/*
 * Functions callable from C, intended to support IEEE arithmetic.
 */
double copysign __P((double, double));
int ilogb __P((double));
double rint __P((double));
long int lrint __P((double));
double scalbn __P((double, int));

/*
 * BSD math library entry points
 */
double expm1 __P((double));
double log1p __P((double));

/*
 * Reentrant version of gamma & lgamma; passes signgam back by reference
 * as the second argument; user must allocate space for signgam.
 */
double gamma_r __P((double, int *));
double lgamma_r __P((double, int *));

/* ieee style elementary functions */
double __ieee754_sqrt __P((double));			
double __ieee754_acos __P((double));			
double __ieee754_acosh __P((double));			
double __ieee754_log __P((double));			
double __ieee754_atanh __P((double));			
double __ieee754_asin __P((double));			
double __ieee754_atan2 __P((double,double));			
double __ieee754_exp __P((double));
double __ieee754_cosh __P((double));
double __ieee754_fmod __P((double,double));
double __ieee754_pow __P((double,double));
double __ieee754_lgamma_r __P((double,int *));
double __ieee754_gamma_r __P((double,int *));
double __ieee754_lgamma __P((double));
double __ieee754_gamma __P((double));
double __ieee754_log10 __P((double));
double __ieee754_sinh __P((double));
double __ieee754_hypot __P((double,double));
double __ieee754_j0 __P((double));
double __ieee754_j1 __P((double));
double __ieee754_y0 __P((double));
double __ieee754_y1 __P((double));
double __ieee754_jn __P((int,double));
double __ieee754_yn __P((int,double));
double __ieee754_remainder __P((double,double));
int    __ieee754_rem_pio2 __P((double,double*));
#ifdef _SCALB_INT
double __ieee754_scalb __P((double,int));
#else
double __ieee754_scalb __P((double,double));
#endif

/* fdlibm kernel function */
double __kernel_standard __P((double,double,int));	
double __kernel_sin __P((double,double,int));
double __kernel_cos __P((double,double));
double __kernel_tan __P((double,double,int));
int    __kernel_rem_pio2 __P((double*,double*,int,int,int,const int*));

#endif /* _MATH_H_ */
